using Microsoft.Extensions.Configuration;
using System.Text.Json;
using System.Text.Json.Serialization;
using View_by_Distance.Shared.Models;

namespace View_by_Distance.Windows.Models;

public record AppSettings(ResultSettings ResultSettings,
                          MetadataSettings MetadataSettings,
                          WindowsSettings WindowsSettings)
{

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, AppSettingsSourceGenerationContext.Default.AppSettings);
        return result;
    }

    private static void Verify(AppSettings appSettings)
    {
        if (appSettings.WindowsSettings.MaxDegreeOfParallelism > Environment.ProcessorCount)
            throw new Exception("MaxDegreeOfParallelism must be =< Environment.ProcessorCount!");
    }

    public static AppSettings Get(IConfigurationRoot configurationRoot)
    {
        AppSettings result;
#pragma warning disable IL3050, IL2026
        ResultSettings? resultSettings = configurationRoot.GetSection(nameof(ResultSettings)).Get<ResultSettings>();
        MetadataSettings? metadataSettings = configurationRoot.GetSection(nameof(MetadataSettings)).Get<MetadataSettings>();
        WindowsSettings? WindowsSettings = configurationRoot.GetSection(nameof(WindowsSettings)).Get<WindowsSettings>();
#pragma warning restore IL3050, IL2026
        if (resultSettings is null || metadataSettings is null || WindowsSettings?.Company is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
        result = new(resultSettings, metadataSettings, WindowsSettings);
        Verify(result);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(AppSettings))]
internal partial class AppSettingsSourceGenerationContext : JsonSerializerContext
{
}